/*
 * statfs() emulation for MiNT/TOS
 *
 * Written by Adrian Ashley (adrian@secret.uucp)
 * and placed in the public domain.
 */

#include <errno.h>
#include <stat.h>
#include <osbind.h>
#include <unistd.h> /* for chdir, getcwd */
#include <limits.h> /* for PATH_MAX */
#ifdef __TURBOC__
#include <sys\statfs.h>
#else
#include <sys/statfs.h>
#endif

extern int __mint;

int statfs(path, buf)
  char *path;
  struct statfs *buf;
{
  int r;
  _DISKINFO free;
  struct stat statbuf;

  if (!buf || !path)
  {
    errno = EFAULT;
    return -1;
  }

  r = stat(path, &statbuf);

  if (r == -1)
    return -1;

  if ((__mint >= 99) && (statbuf.st_dev >= 32))
  {
    /* Hack by HPP 02/06/1993: since MiNT 0.99 only returns     */
    /* valid dfree info for pseudo-drives if they are the       */
    /* current directory, change directories for the occasion.  */
    char oldpath[PATH_MAX];
    
    if (getcwd(oldpath, PATH_MAX) != NULL)
    {
      chdir(path);
      Dfree(&free, statbuf.st_dev + 1);
      chdir(oldpath);
    }
    else
      Dfree(&free, statbuf.st_dev + 1);
  }
  else
    Dfree(&free, statbuf.st_dev + 1);

  buf->f_type = 0;
  buf->f_bsize = free.b_secsiz * free.b_clsiz;
  buf->f_blocks = free.b_total;
  buf->f_bfree = buf->f_bavail = free.b_free;
  buf->f_files = buf->f_ffree = buf->f_fsid.val[0] = buf->f_fsid.val[1] = -1L;

  return 0;
}

#ifdef TEST

#include <stdio.h>

main(int argc, char **argv)
{
  int i = 0, r;
  register char *p;
  struct statfs stbuf;

  while (--argc)
  {
    p = argv[++i];

    r = statfs(p, &stbuf);
    if (r == -1)
      perror(p);
    else
      printf("statfs(`%s'): %ld free bytes\n", p,
	(long)(stbuf.f_bfree * stbuf.f_bsize));
  }
  return 0;
}

#endif
